/**
 * configuration for grunt tasks
 * @module Gruntfile
 */

module.exports = function(grunt) {
  /** load tasks */
  require('load-grunt-tasks')(grunt);

  /* ==========================================================================
  // file config
  ========================================================================== */

  /** paths to file on server */
  var files = {

    /** meta / non-script files */
    meta: [
      'README.md',
      '**/README.md',
      'Procfile',
      'TODO.md',
      'package.json',
      '.gitignore',
      '*.sublime-project',
      '*.sublime-workspace',
      '*.iml',
      '.idea',
    ],

    /** js scripts */
    js: [
      'src/*',
    ],

    /** array of all paths */
    all: []
  };

  /** add meta files to `files.all` */
  files.meta.forEach(function(file) {
    files.all.push(file);
  });

  /** add js files to `files.all` */
  files.js.forEach(function(file) {
    files.all.push(file);
  });

  /** ignore meta files on js */
  files.meta.forEach(function(file) {
    files.js.push('!' + file);
  });

/* ==========================================================================
// task config
========================================================================== */

  /** all grunt tasks */
  var tasks = {

    /** metadata inside `package.json` file */
    pkg: grunt.file.readJSON('package.json'),

    /** lint javascript tool */
    jshint: {

      /** paths to files */
      files: files.js,

      /** options for jshint task */
      options: {

        /** options here to override JSHint defaults */
        globals: {
          console: true,
          module: true
        }
      }
    },

    /** build TODO.md from inline comments */
    todos: {
      src: {
        /** options for plugin */
        options: {
          priorities: {
            low: /TODO/,
            med: /FIXME/,
            high: null
          },
          reporter: {
            /** put at the top of the generated file */
            header: function() {
              return '-- BEGIN TASK LIST --\n\n';
            },

            /** flow for each file */
            fileTasks: function(file, tasks, options) {
              /** skip if no tasks or checking Gruntfile */
              if (!tasks.length || file && file === 'Gruntfile.js') {
                return '';
              }

              var result = '* ' + file + ' (' + tasks.length + ')\n\n';

              /** iterate over tasks, add data */
              tasks.forEach(function(task) {
                result += '    [' + task.lineNumber + ' - ' +
                  task.priority + '] ' + task.line.trim() + '\n';

                result += '\n';
              });

              return result;
            },
            /** put at the bottom of the generated file */
            footer: function() {
              return '-- END TASK LIST --\n';
            }
          }
        },
        files: {
          'TODO.md': files.js,
        }
      }
    },

    bump: {
      options: {
        files: [
          'package.json',
        ],
        updateConfigs: [],
        commit: false, // true,
        commitMessage: 'Release v%VERSION%',
        commitFiles: ['package.json'],
        createTag: false, // true,
        tagName: 'v%VERSION%',
        tagMessage: 'Version %VERSION%',
        push: false, // true,
        // pushTo: 'upstream',
        gitDescribeOptions: '--tags --always --abbrev=1 --dirty=-d',
        globalReplace: false,
        prereleaseName: false,
        regExp: false
      }
    },

    release: {
      options: {
        bump: false, //default: true
        changelog: false, //default: false
        changelogText: '<%= version %>\n', //default: '### <%= version %> - <%= grunt.template.today("yyyy-mm-dd") %>\n'
        file: 'package.json', // default: package.json
        add: true, //default: true
        commit: true, //default: true
        tag: true, //default: true
        push: false, //default: true
        // pushTags: true, //default: true
        npm: false, //default: true
        npmtag: false, //default: no tag
        indentation: '', //default: '  ' (two spaces)
        // folder: 'folder/to/publish/to/npm', //default project root
        tagName: '<%= version %>', //default: '<%= version %>'
        commitMessage: 'release - v<%= version %>', //default: 'release <%= version %>'
        tagMessage: 'release - v<%= version %>', //default: 'Version <%= version %>',
        beforeBump: [], // optional grunt tasks to run before file versions are bumped
        afterBump: [], // optional grunt tasks to run after file versions are bumped
        beforeRelease: [
          'shell:gitAddAll'
        ], // optional grunt tasks to run after release version is bumped up but before release is packaged
        afterRelease: [], // optional grunt tasks to run after release is packaged
        updateVars: [], // optional grunt config objects to update (this will update/set the version property on the object specified)
      },
    },

    /** shell commands */
    shell: {
      gitAddAll: {
        command: 'git add -A',
      },
      pushProduction: {
        command: 'git push origin master && git push heroku master',
      },
      pushStaging: {
        command: 'git push origin staging && git push staging staging:master',
      },
    },

    /** karma testing config */
    karma {
      unit: {
        configFile: 'test/karma.conf.js',
      },
    },
  };

  /** merge files and task config, initialize grunt config */
  grunt.initConfig(grunt.util._.extend(tasks, files));

  /** default grunt task, ran with `grunt` */
  grunt.registerTask('default', function() {
    throw Error('No default task set.');
  });

  /** build server files */
  grunt.registerTask('build', [
    'jshint',
    'todos'
  ]);

  /** documents the server, bumps the version, tags it */
  grunt.registerTask('prepareDeploy', [
    'doc',
    'shell:gitAddAll',
    'bump',
    'release',
  ]);

  /** pushes code to remote repo & live server */
  grunt.registerTask('deploy', 'This task pushes the code to the remote repos.', function(path) {
    var
      args = Array.prototype.slice.call(arguments),
      environment = args[0];

    if (args.length > 1 || (args.length === 1 && ['staging', 'production'].indexOf(args[0]) === -1)) {
      throw new Error('Can only deploy `staging` or `production`');
    }

    if (!args.length) {
      // deploy to staging and production
      console.log('deploying to staging and production');
      grunt.task.run('shell:pushProduction');
      grunt.task.run('shell:pushStaging');
    } else if (args[0] === 'staging') {
      console.log('deploying to staging');
      grunt.task.run('shell:pushStaging');
    } else if (args[0] === 'production') {
      console.log('deploying to production');
      grunt.task.run('shell:pushProduction');
    }
  });
};